@@ -91,6 +91,22 @@ sub getOrder
 
 =pod
 
+=head2 getOrder
+
+Get order to do handler.
+
+=cut
+
+sub useprevious()
+{
+    my $self = shift;
+    my $conf = $self->{_config};
+
+    return defined($conf->{useprevious}) && $conf->{useprevious} == 'on' ? 1 : 0;
+}
+
+=pod
+
 =head2 pre_bind
 
 This method is called bofore L<LISM> do the bind operation.
@@ -300,7 +316,7 @@ sub log
     if (Encode::is_utf8($p{'message'})) {
         $p{'message'} = encode('utf8', $p{'message'});
     }
-    $p{'message'} = '['.ref($self).'] '.$p{'message'};
+    $p{'message'} = uc($p{'level'}).' ['.ref($self).'] '.$p{'message'};
 
     if ($^O ne 'MSWin32') {
         openlog('LISM', 'pid', 'local4');
@@ -415,45 +431,69 @@ sub _searchLdap
 sub _searchLism
 {
     my $self = shift;
-    my ($lismopts, $filter) = @_;
+    my ($lismopts, $filters) = @_;
     my $base = $lismopts->{base};
     my $scope = 2;
     my $attr;
-    my @values = ('');
+    my @values = ();
 
-    $filter =~ s/^["']//;
-    $filter =~ s/["']$//;
-    if ($filter =~ /^\(?dn=/) {
-        ($base) = ($filter =~ /^\(?dn=(.*)\)?$/);
-        $scope = 0;
-        $filter = '(objectClass=*)';
+    if (defined($lismopts->{scope})) {
+        if ($lismopts->{scope} eq 'base') {
+            $scope = 0;
+        } elsif ($lismopts->{scope} eq 'one') {
+            $scope = 1;
+        }
     }
+
     if ($lismopts->{attr} eq 'parentrdn') {
         $attr = 'dn';
     } else {
         $attr = $lismopts->{attr};
     }
 
-    my ($rc, @entries) = $self->{lism}->search($base, $scope, 0, 0, 0, $filter, 0, $attr);
-    if ($rc) {
-        $self->log(level => 'err', message => "lism search by $filter failed($rc)");
+    foreach my $filter (split(/, */, $filters)) {
+        $filter =~ s/^["']//;
+        $filter =~ s/["']$//;
+        if ($filter =~ /^\(?dn=/) {
+            ($base) = ($filter =~ /^\(?dn=(.*)\)?$/);
+            $scope = 0;
+            $filter = '(objectClass=*)';
+        }
 
-        return undef;
+        my ($rc, @entries) = $self->{lism}->search($base, $scope, 0, 0, 0, encode('utf8', $filter), 0, $attr);
+        if ($rc) {
+            $self->log(level => 'err', message => "lism search by $filter failed($rc)");
+
+            return undef;
+        }
+
+        if (@entries) {
+            my $num = 1;
+            foreach my $entryStr (@entries) {
+                if (defined($lismopts->{sizelimit}) && $num > $lismopts->{sizelimit}) {
+                    last;
+                }
+
+                my @tmpvals;
+                if ($lismopts->{attr} eq 'dn') {
+                    @tmpvals = ($entryStr =~ /^[^\n]+/);
+                } elsif ($lismopts->{attr} eq 'parentrdn') {
+                    @tmpvals = ($entryStr =~ /^[^,]+,([^,]+)/);
+                } else {
+                    @tmpvals = ($entryStr =~ /^$lismopts->{attr}: (.*)$/gmi);
+                }
+
+                if (@tmpvals && defined($tmpvals[0]) && $tmpvals[0] !~ /^ *$/) {
+                    push(@values, @tmpvals);
+                }
+                $num++;
+            }
+            last;
+        }
     }
 
-    if (@entries) {
-        my $entryStr = $entries[0];
-        if ($lismopts->{attr} eq 'dn') {
-            @values = ($entryStr =~ /^[^\n]+/);
-        } elsif ($lismopts->{attr} eq 'parentrdn') {
-            @values = ($entryStr =~ /^[^,]+,([^,]+)/);
-        } else {
-            @values = ($entryStr =~ /^$lismopts->{attr}: (.*)$/mi);
-        }
-
-        if (!defined($values[0])) {
-            @values = ('');
-        }
+    if (!@values || !defined($values[0])) {
+        @values = ('');
     }
 
     return @values;
