@@ -41,7 +41,6 @@ sub post_search
     my $self = shift;
     my ($entriesp) = @_;
     my $conf = $self->{_config};
-    my $fd;
 
     my $match = 0;
     foreach my $rule (@{$conf->{check}}) {
@@ -54,25 +53,8 @@ sub post_search
         return LDAP_SUCCESS;
     }
 
-    if (defined($conf->{file})) {
-        if ($conf->{file}[0] eq 'stdout') {
-            $fd = *STDOUT;
-        } elsif (!open($fd, ">> $conf->{file}[0]")) {
-            $self->log(level => 'err', message => "Can't open $conf->{file}[0]");
-            return LDAP_OTHER;
-        }
-    } elsif (defined($conf->{command})) {
-        if (!open($fd, "|$conf->{command}[0]")) {
-            $self->log(level => 'err', message => "Can't open $conf->{command}[0]");
-            return LDAP_OTHER;
-        }
-    }
-
     for (my $i = 0; $i < @{$entriesp}; $i++) {
-        $self->_checkValues(${$entriesp}[$i], 'search', $fd);
-    }
-    if (!defined($conf->{file}) || $conf->{file}[0] ne 'stdout') {
-        close($fd);
+        $self->_checkValues(${$entriesp}[$i], 'search');
     }
 
     return LDAP_SUCCESS;
@@ -89,21 +71,6 @@ sub pre_modify
     my $self = shift;
     my ($dnp, $listp, $oldentryp) = @_;
     my $conf = $self->{_config};
-    my $fd;
-
-    if (defined($conf->{file})) {
-        if ($conf->{file}[0] eq 'stdout') {
-            $fd = *STDOUT;
-        } elsif (!open($fd, ">> $conf->{file}[0]")) {
-            $self->log(level => 'err', message => "Can't open $conf->{file}[0]");
-            return LDAP_OTHER;
-        }
-    } elsif (defined($conf->{command})) {
-        if (!open($fd, "|$conf->{command}[0]")) {
-            $self->log(level => 'err', message => "Can't open $conf->{command}[0]");
-            return LDAP_OTHER;
-        }
-    }
 
     my $entryStr = "dn: ${$dnp}\n";
     my @list = @{$listp};
@@ -125,10 +92,7 @@ sub pre_modify
         }
     }
 
-    my $rc = $self->_checkValues($entryStr, 'modify', $fd, 'pre', $oldentryp ? ${$oldentryp} : '');
-    if (!defined($conf->{file}) || $conf->{file}[0] ne 'stdout') {
-        close($fd);
-    }
+    my $rc = $self->_checkValues($entryStr, 'modify', 'pre', $oldentryp ? ${$oldentryp} : '');
 
     return $rc;
 }
@@ -144,26 +108,8 @@ sub pre_add
     my $self = shift;
     my ($dnp, $entryStrp) = @_;
     my $conf = $self->{_config};
-    my $fd;
 
-    if (defined($conf->{file})) {
-        if ($conf->{file}[0] eq 'stdout') {
-            $fd = *STDOUT;
-        } elsif (!open($fd, ">> $conf->{file}[0]")) {
-            $self->log(level => 'err', message => "Can't open $conf->{file}[0]");
-            return LDAP_OTHER;
-        }
-    } elsif (defined($conf->{command})) {
-        if (!open($fd, "|$conf->{command}[0]")) {
-            $self->log(level => 'err', message => "Can't open $conf->{command}[0]");
-            return LDAP_OTHER;
-        }
-    }
-
-    my $rc = $self->_checkValues("dn: ${$dnp}\n${$entryStrp}[0]", 'add', $fd);
-    if (!defined($conf->{file}) || $conf->{file}[0] ne 'stdout') {
-        close($fd);
-    }
+    my $rc = $self->_checkValues("dn: ${$dnp}\n${$entryStrp}[0]", 'add');
 
     return $rc;
 }
@@ -207,7 +153,7 @@ sub post_modify
         }
     }
 
-    return $self->_checkValues($entryStr, 'modify', undef, 'post', $oldentry);
+    return $self->_checkValues($entryStr, 'modify', 'post', $oldentry);
 }
 
 =head2 post_add($dnp, $entryStrp)
@@ -222,7 +168,7 @@ sub post_add
     my ($dnp, $entryStrp) = @_;
     my $conf = $self->{_config};
 
-    return $self->_checkValues("dn: ${$dnp}\n${$entryStrp}[0]", 'add', undef, 'post');
+    return $self->_checkValues("dn: ${$dnp}\n${$entryStrp}[0]", 'add', 'post');
 }
 
 =head2 post_delete($dnp)
@@ -237,13 +183,13 @@ sub post_delete
     my ($dnp, $null, $oldentryp) = @_;
     my $conf = $self->{_config};
 
-    return $self->_checkValues("dn: ${$dnp}\n", 'delete', undef, 'post', $oldentryp ? ${$oldentryp} : '');
+    return $self->_checkValues("dn: ${$dnp}\n", 'delete', 'post', $oldentryp ? ${$oldentryp} : '');
 }
 
 sub _checkValues
 {
     my $self = shift;
-    my ($entryStr, $func, $fd, $type, $oldentry) = @_;
+    my ($entryStr, $func, $type, $oldentry) = @_;
     my $conf = $self->{_config};
     my $rc = LDAP_SUCCESS;
 
@@ -273,7 +219,7 @@ sub _checkValues
                     if (!defined($mrc)) {
                         return LDAP_OTHER;
                     } elsif (!$mrc) {
-                        print $fd encode('utf8', "$timestamp: Maximum number of entries exceeded($dn): $error\n");
+                        $self->_perror("$timestamp: Maximum number of entries exceeded($dn): $error");
                         return LDAP_ADMIN_LIMIT_EXCEEDED;
                     }
                 }
@@ -315,7 +261,7 @@ sub _checkValues
                     if (!defined($mrc)) {
                         return LDAP_OTHER;
                     } elsif (!$mrc) {
-                        print $fd encode('utf8', "$timestamp: Maximum number of entries exceeded($dn): $error\n");
+                        $self->_perror("$timestamp: Maximum number of entries exceeded($dn): $error");
                         return LDAP_SIZELIMIT_EXCEEDED;
                     }
                 }
@@ -324,27 +270,27 @@ sub _checkValues
                 next;
             }
             if (defined($cattr->{required}) && $cattr->{required}[0] eq 'on' && !@values) {
-                print $fd encode('utf8', "$timestamp: $attr in $dn is required value\n");
+                $self->_perror("$timestamp: $attr in $dn is required value");
                 $rc = LDAP_CONSTRAINT_VIOLATION;
             }
             foreach my $value (@values) {
                 if (defined($cattr->{minlen}) && length($value) < $cattr->{minlen}[0]) {
-                    print $fd encode('utf8', "$timestamp: $attr=$value in $dn is too short\n");
+                    $self->_perror("$timestamp: $attr=$value in $dn is too short");
                     $rc = LDAP_CONSTRAINT_VIOLATION;
                 }
                 if (defined($cattr->{maxlen}) && length($value) > $cattr->{maxlen}[0]) {
-                    print $fd encode('utf8', "$timestamp: $attr=$value in $dn is too long\n");
+                    $self->_perror("$timestamp: $attr=$value in $dn is too long");
                     $rc = LDAP_CONSTRAINT_VIOLATION;
                 }
                 if (defined($cattr->{regexp}) && $value !~ /$cattr->{regexp}[0]/) {
-                    print $fd encode('utf8', "$timestamp: $attr=$value in $dn is invalid: regular expression is $cattr->{regexp}[0]\n");
+                    $self->_perror("$timestamp: $attr=$value in $dn is invalid: regular expression is $cattr->{regexp}[0]");
                     $rc = LDAP_CONSTRAINT_VIOLATION;
                 }
                 if (defined($cattr->{function})) {
                     my $ecode = 0;
                     eval "\$ecode = $cattr->{function}[0](\$value)";
                     if (!$ecode) {
-                        print $fd encode('utf8', "$timestamp: $attr=$value in $dn is invalid: function is $cattr->{function}[0]($ecode)\n");
+                        $self->_perror("$timestamp: $attr=$value in $dn is invalid: function is $cattr->{function}[0]($ecode)");
                         $rc = LDAP_CONSTRAINT_VIOLATION;
                     }
                 }
@@ -356,7 +302,7 @@ sub _checkValues
                     }
                     my @vals = $self->_searchLism($opts, $filter);
                     if (!@vals || !$vals[0]) {
-                        print $fd encode('utf8', "$timestamp: $attr=$value in $dn is invalid: value doesn't exist in data\n");
+                        $self->_perror("$timestamp: $attr=$value in $dn is invalid: value doesn't exist in data");
                         $rc = LDAP_CONSTRAINT_VIOLATION;
                     }
                 }
@@ -370,7 +316,7 @@ sub _checkValues
                     $filter =~ s/\%i/$id/g;
                     my @vals = $self->_searchLism($opts, $filter);
                     if (@vals && $vals[0]) {
-                        print $fd encode('utf8', "$timestamp: $attr=$value in $dn is invalid: value already exist in data\n");
+                        $self->_perror("$timestamp: $attr=$value in $dn is invalid: value already exist in data");
                         $rc = LDAP_CONSTRAINT_VIOLATION;
                     }
                 }
@@ -493,7 +439,7 @@ sub _updateCurrentEntries
                     $currentvals{$service} = $current + ${$opts->{increment}}{$service};
                 }
             }
-            if (($func eq 'modify' || $func eq 'delete') && @delvals) {
+            if (($func eq 'modify' || $func eq 'delete') && @delvals && $delvals[0] !~ /^ *$/) {
                 my $idattr = defined($opts->{uid}) ? $opts->{uid} : $opts->{service};
                 my @ids = ($entryStr =~ /^$idattr: (.+)$/gmi);
                 foreach my $value (@delvals) {
@@ -541,16 +487,46 @@ sub _updateCurrentEntries
             @list = ('REPLACE', $opts->{current}, $current);
         }
 
-        $rc = $self->{lism}->modify($base, @list);
-        if ($rc) {
-            $self->log(level => 'err', message => "modifying current number($base) failed($rc)");
-            return undef;
+        if (@list) {
+            $rc = $self->{lism}->modify($base, @list);
+            if ($rc) {
+                $self->log(level => 'err', message => "modifying current number($base) failed($rc)");
+                return undef;
+            }
         }
     }
 
     return 1;
 }
 
+sub _perror
+{
+    my $self = shift;
+    my ($message) = @_;
+    my $conf = $self->{_config};
+    my $fd;
+
+    if (defined($conf->{file})) {
+        if ($conf->{file}[0] eq 'stdout') {
+            $fd = *STDOUT;
+        } elsif (!open($fd, ">> $conf->{file}[0]")) {
+            $self->log(level => 'err', message => "Can't open $conf->{file}[0]: $!");
+            return LDAP_OTHER;
+        }
+    } elsif (defined($conf->{command})) {
+        if (!open($fd, "|$conf->{command}[0]")) {
+            $self->log(level => 'err', message => "Can't open $conf->{command}[0]: $!");
+            return LDAP_OTHER;
+        }
+    }
+
+    print $fd encode('utf8', "$message\n");
+
+    if (!defined($conf->{file}) || $conf->{file}[0] ne 'stdout') {
+        close($fd);
+    }
+}
+
 sub _checkConfig
 {
     my $self = shift;
