@@ -12,6 +12,7 @@ if ($^O ne 'MSWin32') {
     eval "use Sys::Syslog";
 }
 
+our $controlAttr = 'lismControl';
 our $maxLoopCount = 10;
 
 =head1 NAME
@@ -173,6 +174,11 @@ sub search
     my $conf = $self->{_config};
     my @match_entries = ();
 
+    if ($filterStr =~ /^\(&\($controlAttr=[^\)]+\)/i) {
+        $filterStr =~ s/^\(&\($controlAttr=[^\)]+\)//;
+        $filterStr =~ s/\)$//;
+    }
+
     my $filter = Net::LDAP::Filter->new($filterStr);
     if (!defined($filter)) {
         return (LDAP_FILTER_ERROR, ());
@@ -288,7 +294,7 @@ sub compare
         }
 
         if ($retry >= $conf->{connection}[0]->{retry}[0]) {
-            return (LDAP_SERVER_DOWN, ());
+            return LDAP_SERVER_DOWN;
         }
         sleep $conf->{connection}[0]->{interval}[0];
         $retry++;
@@ -343,7 +349,7 @@ sub modify
         }
 
         if ($retry >= $conf->{connection}[0]->{retry}[0]) {
-            return (LDAP_SERVER_DOWN, ());
+            return LDAP_SERVER_DOWN;
         }
         sleep $conf->{connection}[0]->{interval}[0];
         $retry++;
@@ -386,7 +392,7 @@ sub add
         }
 
         if ($retry >= $conf->{connection}[0]->{retry}[0]) {
-            return (LDAP_SERVER_DOWN, ());
+            return LDAP_SERVER_DOWN;
         }
         sleep $conf->{connection}[0]->{interval}[0];
         $retry++;
@@ -414,6 +420,7 @@ sub modrdn
     my $self = shift;
     my ($dn, $newrdn, $delFlag) = @_;
     my $rc;
+    my $error;
     my $entry;
 
     my ($rdn_attr, $superior) = ($dn =~ /^([^=]*)=[^,]*,(.*)$/);
@@ -432,16 +439,16 @@ sub modrdn
         $entry =~ s/^userpassword: .*$/userpassword: $passwd/mi;
     }
 
-    $rc = $self->add("$newrdn,$superior", $entry);
+    ($rc, $error) = $self->add("$newrdn,$superior", $entry);
     if ($rc) {
-        return $rc;
+        return ($rc, $error);
     }
 
     if ($delFlag) {
-        $rc = $self->delete($dn);
+        ($rc, $error) = $self->delete($dn);
     }
 
-    return $rc;
+    return ($rc, $error);
 }
 
 =pod
@@ -466,7 +473,7 @@ sub delete
         }
 
         if ($retry >= $conf->{connection}[0]->{retry}[0]) {
-            return (LDAP_SERVER_DOWN, ());
+            return LDAP_SERVER_DOWN;
         }
         sleep $conf->{connection}[0]->{interval}[0];
         $retry++;
@@ -476,10 +483,10 @@ sub delete
     if ($rc) {
         return $rc;
     }
+    my @pkeyarr = @{$pkeys};
 
     if ($obj->{conf}->{subcontainer}) {
-        my $level = split(/,/, ($dn =~ /^(.*),?$self->{suffix}$/i)[0]);
-        if ($obj->{conf}->{subcontainer}[0]->{level} == $level &&
+        if ($dn !~ /,$obj->{conf}->{subcontainer}[0]->{rdn}[0],/i &&
             $dn =~ /^$obj->{conf}->{subcontainer}[0]->{rdn}[0],/i) {
             return LDAP_UNWILLING_TO_PERFORM;
         }
@@ -506,6 +513,54 @@ sub delete
 
 =pod
 
+=head2 move
+
+This method is called when L<LISM> do the move operation.
+Returns 0 if it completes successfully.
+
+=cut
+
+sub move
+{
+    my $self = shift;
+    my ($dn, $parentdn) = @_;
+    my $conf = $self->{_config};
+
+    my $retry = 0;
+    while (1) {
+        if (!$self->_getConnect()) {
+            last;
+        }
+
+        if ($retry >= $conf->{connection}[0]->{retry}[0]) {
+            return LDAP_SERVER_DOWN;
+        }
+        sleep $conf->{connection}[0]->{interval}[0];
+        $retry++;
+    }
+
+    my ($rc, $obj, $pkeys) = $self->_getObject($dn);
+    if ($rc) {
+        return $rc;
+    }
+
+    my $newobj;
+    my $newpkeys;
+    my ($rdn) = ($dn =~ /^([^,]+),/);
+    ($rc, $newobj, $newpkeys) = $self->_getObject("$rdn,$parentdn");
+    if ($rc) {
+        return $rc;
+    }
+
+    if ($obj != $newobj) {
+        return LDAP_UNWILLING_TO_PERFORM;
+    }
+
+    return $self->_objMove($obj, $pkeys, $dn, $newpkeys, "$rdn,$parentdn");
+}
+
+=pod
+
 =head2 log(level, message)
 
 log message to syslog.
@@ -521,10 +576,10 @@ sub log
     if (Encode::is_utf8($p{'message'})) {
         $p{'message'} = encode('utf8', $p{'message'});
     }
-    $p{'message'} = '['.ref($self).'] '.$p{'message'};
+    $p{'message'} = uc($p{'level'}).' ['.ref($self).'] '.$p{'message'};
 
     if ($^O ne 'MSWin32') {
-        openlog('LISM', 'pid', 'local4');
+        openlog('LISM', 'pid', $self->{lism}->{_config}->{syslogfacility});
         setlogmask(Sys::Syslog::LOG_UPTO(Sys::Syslog::xlate($conf->{sysloglevel})));
         syslog($p{'level'}, $p{'message'});
         closelog();
@@ -779,11 +834,11 @@ sub _checkConfig
 
     # connection
     if (!defined($conf->{connection}[0]->{retry})
-        || $conf->{connection}[0]->{retry}[0] !~ /[0-9]*/) {
+        || $conf->{connection}[0]->{retry}[0] !~ /^[0-9]+$/) {
         $conf->{connection}[0]->{retry}[0] = 0;
     }
     if (!defined($conf->{connection}[0]->{interval})
-        || $conf->{connection}[0]->{interval}[0] !~ /[0-9]*/) {
+        || $conf->{connection}[0]->{interval}[0] !~ /^[0-9]+$/) {
         $conf->{connection}[0]->{interval}[0] = 10;
     }
 
@@ -796,6 +851,7 @@ sub _checkConfig
 
             $self->{object}{$oname}->{name} = $oname;
             $self->{object}{$oname}->{conf} = $oconf;
+            $self->{object}{$oname}->{recursive} = 0;
 
             # set container
             if (!defined($oconf->{container})) {
@@ -812,7 +868,22 @@ sub _checkConfig
                 push(@{$self->{object}{$coname}->{child}}, $self->{object}{$oname});
                 $self->{object}{$oname}->{parent} = $coname;
             } elsif (defined($oconf->{container}[0]->{oname})) {
-                my $poname = $oconf->{container}[0]->{oname}[0];
+                my $poname;
+                if ($oconf->{container}[0]->{oname}[0] eq $oname) {
+                    if (defined($oconf->{container}[0]->{parent})) {
+                        $poname = $oconf->{container}[0]->{parent}[0];
+                    } else {
+                        $poname = 'top_'.$oname.'_container';
+                        if (!defined($self->{object}{$poname})) {
+                            $self->{object}{$poname}->{entrystr} = '';
+                            # normalize suffix
+                            ($self->{object}{$poname}->{suffix} = $self->{suffix}) =~ tr/A-Z/a-z/;
+                        }
+                    }
+                    $self->{object}{$oname}->{recursive} = 1;
+                } else {
+                    $poname = $oconf->{container}[0]->{oname}[0];
+                }
                 $self->{object}{$oname}->{parent} = $poname;
                 if (!defined($self->{object}{$poname}->{child})) {
                     $self->{object}{$poname}->{child} = [];
@@ -925,12 +996,9 @@ sub _getTreeLevel
     }
 
     if (defined($obj->{conf}->{subcontainer})) {
-        $obj->{conf}->{subcontainer}[0]->{level} = $current;
         $current++;
     }
 
-    $obj->{level} = $current;
-
     if (!defined($obj->{child})) {
         return $current;
     }
@@ -990,8 +1058,7 @@ sub _getObject
     my @objs = ($obj);
     my @pkeyarr = ();
     my $key;
-    for (my $base = $obj->{suffix}; defined($obj->{child});
-        @objs = @{$obj->{child}}, $base = pop(@rdn_list).','.$base) {
+    for (my $base = $obj->{suffix};1; $base = pop(@rdn_list).','.$base) {
         my $rc;
         my $entry;
 
@@ -1004,7 +1071,7 @@ sub _getObject
             if (defined($objs[$i]->{conf}->{subcontainer})) {
                 my $rdn = $objs[$i]->{conf}->{subcontainer}[0]->{rdn}[0];
                 if ($base =~ /^$rdn,/i) {
-                    $key = $pkeyarr[$#pkeyarr];
+                    $key = undef;
                     $entry = "subcontainer";
                     last;
                 }
@@ -1017,12 +1084,21 @@ sub _getObject
             }
         }
         push(@pkeyarr, $key);
-        if (!$entry) {
+
+        if (!$entry && (!$obj->{recursive} || !defined($obj->{child}))) {
             return (LDAP_NO_SUCH_OBJECT, $obj, []);
         }
         if (!@rdn_list) {
             last;
         }
+
+        if ($obj->{recursive}) {
+            @objs = ($obj);
+        } elsif (defined($obj->{child})) {
+            @objs = @{$obj->{child}};
+        } else {
+            last;
+        }
     }
 
     if (defined($obj->{child})) {
@@ -1038,6 +1114,8 @@ sub _getObject
                 return (LDAP_SUCCESS, $child, \@pkeyarr);
             }
         }
+    } elsif ($obj->{recursive}) {
+        return (LDAP_SUCCESS, $obj, \@pkeyarr);
     }
 
     return (LDAP_NO_SUCH_OBJECT, $obj, []);
@@ -1086,9 +1164,7 @@ sub _baseSearch
     }
 
     if (defined($oconf->{subcontainer})) {
-        my $level = split(/,/, ($base =~ /^(.*),?$self->{suffix}$/i)[0]);
-
-        if ($oconf->{subcontainer}[0]->{level} == $level) {
+        if ($base !~ /,$oconf->{subcontainer}[0]->{rdn}[0],/i) {
             if ($base !~ /^$oconf->{subcontainer}[0]->{rdn}[0],/i) {
                 return (LDAP_NO_SUCH_OBJECT, undef, undef);
             }
@@ -1097,9 +1173,9 @@ sub _baseSearch
             $suffix =~ s/^[^,]+,//;
             my $entry = $self->buildEntryStr($suffix, $oconf->{subcontainer}[0]);
             if ($self->parseFilter($filter, $entry) && $scope != 1 && $sizeLim) {
-                return (LDAP_SUCCESS, $pkeyarr[$#pkeyarr], $entry);
+                return (LDAP_SUCCESS, undef, $entry);
             } else {
-                return (LDAP_SUCCESS, $pkeyarr[$#pkeyarr], undef);
+                return (LDAP_SUCCESS, undef, undef);
             }
         }
     }
@@ -1150,9 +1226,10 @@ sub _childSearch
 
     my @entries;
     my $keys;
-    my $level = split(/,/, ($base =~ /^(.*),?$self->{suffix}$/i)[0]);
 
-    if (defined($oconf->{subcontainer}) && $oconf->{subcontainer}[0]->{level} == $level) {
+    if (defined($oconf->{subcontainer}) && $base =~ /^$oconf->{subcontainer}[0]->{rdn}[0],/i) {
+        @objs = ($obj);
+    } elsif ($obj->{recursive}) {
         @objs = ($obj);
     } else {
         if (!defined($obj->{child})) {
@@ -1161,14 +1238,15 @@ sub _childSearch
         @objs = @{$obj->{child}};
     }
 
+    RECURSIVE:
     for (my $i = 0; $i < @objs; $i++) {
         my $cbase;
         my @children = ();
         my $ckeys;
 
-        if (defined($objs[$i]->{conf}->{subcontainer}) &&
-            $objs[$i]->{conf}->{subcontainer}[0]->{level} != $level) {
+        if (defined($objs[$i]->{conf}->{subcontainer}) && $base !~ /^(|.+,)$objs[$i]->{conf}->{subcontainer}[0]->{rdn}[0],/i) {
             my $entry = $self->buildEntryStr($base, $objs[$i]->{conf}->{subcontainer}[0]);
+
             if ($self->parseFilter($filter, $entry) && $sizeLim) {
                 push(@match_entries, $entry);
                 push(@match_keys, undef);
@@ -1185,7 +1263,7 @@ sub _childSearch
         push(@match_entries, @entries);
         push(@match_keys, @{$keys});
 
-        if ($scope == 1 || !defined($objs[$i]->{child}) || $rc) {
+        if ($scope == 1 || (!$objs[$i]->{recursive} && !defined($objs[$i]->{child})) || $rc) {
             next;
         }
         $sizeLim = $sizeLim - @match_entries;
@@ -1195,6 +1273,13 @@ sub _childSearch
             return ($rc, \@match_keys, @match_entries);
         }
 
+        if ($objs[$i]->{recursive} && @{$ckeys} == 0) {
+            if (defined($objs[$i]->{child})) {
+                @objs = @{$objs[$i]->{child}};
+                goto RECURSIVE;
+            }
+        }
+
         for (my $j = 0; $j < @{$ckeys}; $j++) {
             my ($childdn) = ($children[$j] =~ /^dn: (.*)\n/);
             if (!$childdn) {
@@ -1235,28 +1320,50 @@ sub _objDelete
     return LDAP_UNWILLING_TO_PERFORM;
 }
 
+sub _objMove
+{
+    return LDAP_UNWILLING_TO_PERFORM;
+}
+
 sub _getParentDn
 {
     my $self = shift;
     my ($obj, $key) = @_;
     my $conf = $self->{_config};
 
+    RECURSIVE:
     if (defined($obj->{suffix})) {
         return $obj->{suffix};
     }
 
-    if (!defined($obj->{parent})) {
-        return undef;
+    my $pobj;
+    if ($obj->{recursive}) {
+        $pobj = $obj;
+    } else {
+        if (!defined($obj->{parent})) {
+            return undef;
+        }
+
+        $pobj = $self->{object}{$obj->{parent}};
+
+        if (defined($pobj->{suffix})) {
+            return $pobj->{suffix};
+        }
     }
 
-    my $pobj = $self->{object}{$obj->{parent}};
-    if (defined($pobj->{suffix})) {
-        return $pobj->{suffix};
-    }
-
-    my ($prdn, $pkey) = $self->_getParentRdn($obj, $key);
+    my ($prdn, $pkey) = $self->_getParentRdn($obj, $key, $pobj);
     if (!$prdn) {
-        return undef;
+        if ($obj->{recursive}) {
+            if (defined($obj->{conf}->{container}[0]->{parent})) {
+                $obj = $self->{object}{$obj->{parent}};
+                goto RECURSIVE;
+            } else {
+                $prdn = $obj->{conf}->{subcontainer}[0]->{rdn}[0];
+                $pobj = $self->{object}{$obj->{parent}};
+            }
+        } else {
+            return undef;
+        }
     }
 
     my $ppdn = $self->_getParentDn($pobj, $pkey);
@@ -1264,7 +1371,7 @@ sub _getParentDn
         return undef;
     }
 
-    if (defined($obj->{conf}->{subcontainer})) {
+    if (defined($obj->{conf}->{subcontainer}) && !$obj->{recursive}) {
         $prdn = "$obj->{conf}->{subcontainer}[0]->{rdn}[0],$prdn";
     }
 
@@ -1276,6 +1383,18 @@ sub _getParentRdn
     return undef;
 }
 
+sub _getPid
+{
+    my $self = shift;
+    my ($pkeys) = @_;
+
+    if (!defined($pkeys)) {
+        return undef;
+    }
+
+    return defined(${$pkeys}[$#{$pkeys}]) || !$#{$pkeys} ? ${$pkeys}[$#{$pkeys}] : ${$pkeys}[$#{$pkeys} - 1];
+}
+
 sub _getStaticValue
 {
     my $self = shift;
@@ -1298,6 +1417,7 @@ sub _containerParse
 {
     my $self = shift;
     my ($str, @conts) = @_;
+    my $cont = defined($conts[$#conts]) || !$#conts ? $conts[$#conts] : $conts[$#conts - 1];
 
     my @nums = ($str =~ /\%c([0-9]+)/g);
     foreach my $num (@nums) {
@@ -1305,7 +1425,7 @@ sub _containerParse
             $str =~ s/\%c$num/$conts[$#conts + 1 - $num]/;
         }
     }
-    $str =~ s/\%c/$conts[$#conts]/g;
+    $str =~ s/\%c/$cont/g;
 
     return $str;
 }
@@ -1356,7 +1476,7 @@ sub _doPlugin
                     }
 
                     my $level = split(/,/, ($entries[$i] =~ /^dn: (.*),?$self->{suffix}\n/i)[0]);
-                    if ($obj->{level} == $level &&
+                    if (@{$args[0]} == $level &&
                         $self->parseFilter($args[3], $entries[$i])) {
                         push(@match_keys, ${$keys}[$i]);
                         push(@match_entries, $entries[$i]);
@@ -1475,6 +1595,32 @@ sub _writeUpdateLog
     return 0;
 }
 
+sub random
+{
+    my $self = shift;
+    my ($len) = @_;
+    my $string;
+
+    my @chars;
+    push(@chars, ('a'..'z'), ('A'..'Z',), (0..9));
+
+    my ($sec, $microsec) = gettimeofday();
+    srand($microsec);
+
+    $string = '';
+
+    for (my $i = 0; $i < 10; $i++) {
+        for (my $j = 0; $j < $len; $j++) {
+            $string .= $chars[int(rand() * @chars)];
+        }
+        if ($string =~ /[a-zA-Z]/ && $string =~ /[0-9]/) {
+            last;
+        }
+    }
+
+    return $string;
+}
+
 =head1 SEE ALSO
 
 L<LISM>
